@@ -3,10 +3,13 @@ from __future__ import unicode_literals
 
 import difflib
 import inspect
+import logging
 import operator
 import re
+import six
 import sys
 
+from collections import OrderedDict
 from functools import wraps, partial
 from types import MethodType
 
@@ -15,16 +18,14 @@ from flask import make_response as original_flask_make_response
 from flask.helpers import _endpoint_from_view_func
 from flask.signals import got_request_exception
 
-from jsonschema import RefResolver, FormatChecker
+from jsonschema import RefResolver
 
 from werkzeug import cached_property
 from werkzeug.datastructures import Headers
 from werkzeug.exceptions import HTTPException, MethodNotAllowed, NotFound, NotAcceptable, InternalServerError
-from werkzeug.http import HTTP_STATUS_CODES
 from werkzeug.wrappers import BaseResponse
 
 from . import apidoc
-from ._compat import OrderedDict
 from .mask import ParseError, MaskError
 from .namespace import Namespace
 from .postman import PostmanCollectionV1
@@ -32,6 +33,7 @@ from .resource import Resource
 from .swagger import Swagger
 from .utils import default_id, camel_to_dash, unpack
 from .representations import output_json
+from ._http import HTTPStatus
 
 RE_RULES = re.compile('(<.*>)')
 
@@ -40,6 +42,8 @@ HEADERS_BLACKLIST = ('Content-Length',)
 
 DEFAULT_REPRESENTATIONS = [('application/json', output_json)]
 
+log = logging.getLogger(__name__)
+
 
 class Api(object):
     '''
@@ -70,6 +74,7 @@ class Api(object):
     :param str default_label: The default namespace label (used in Swagger documentation)
     :param str default_mediatype: The default media type to return
     :param bool validate: Whether or not the API should perform input payload validation.
+    :param bool ordered: Whether or not preserve order models and marshalling.
     :param str doc: The documentation path. If set to a false value, documentation is disabled.
                 (Default to '/')
     :param list decorators: Decorators to attach to every resource
@@ -79,8 +84,8 @@ class Api(object):
     :param bool serve_challenge_on_401: Serve basic authentication challenge with 401
         responses (default 'False')
     :param FormatChecker format_checker: A jsonschema.FormatChecker object that is hooked into
-    the Model validator. A default or a custom FormatChecker can be provided (e.g., with custom
-    checkers), otherwise the default action is to not enforce any format validation.
+        the Model validator. A default or a custom FormatChecker can be provided (e.g., with custom
+        checkers), otherwise the default action is to not enforce any format validation.
     '''
 
     def __init__(self, app=None, version='1.0', title=None, description=None,
@@ -88,7 +93,7 @@ class Api(object):
             contact=None, contact_url=None, contact_email=None,
             authorizations=None, security=None, doc='/', default_id=default_id,
             default='default', default_label='Default namespace', validate=None,
-            tags=None, prefix='',
+            tags=None, prefix='', ordered=False,
             default_mediatype='application/json', decorators=None,
             catch_all_404s=False, serve_challenge_on_401=False, format_checker=None,
             **kwargs):
@@ -104,6 +109,7 @@ class Api(object):
         self.authorizations = authorizations
         self.security = security
         self.default_id = default_id
+        self.ordered = ordered
         self._validate = validate
         self._doc = doc
         self._doc_view = None
@@ -351,14 +357,14 @@ class Api(object):
         return func
 
     def render_root(self):
-        self.abort(404)
+        self.abort(HTTPStatus.NOT_FOUND)
 
     def render_doc(self):
         '''Override this method to customize the documentation page'''
         if self._doc_view:
             return self._doc_view()
         elif not self._doc:
-            self.abort(404)
+            self.abort(HTTPStatus.NOT_FOUND)
         return apidoc.ui_for(self)
 
     def default_endpoint(self, resource, namespace):
@@ -412,11 +418,8 @@ class Api(object):
         for resource, urls, kwargs in ns.resources:
             self.register_resource(ns, resource, *self.ns_urls(ns, urls), **kwargs)
         # Register models
-        for name, definition in ns.models.items():
+        for name, definition in six.iteritems(ns.models):
             self.models[name] = definition
-        # Register error handlers
-        for exception, handler in ns.error_handlers.items():
-            self.error_handlers[exception] = handler
 
     def namespace(self, *args, **kwargs):
         '''
@@ -424,6 +427,7 @@ class Api(object):
 
         :returns Namespace: a new namespace instance
         '''
+        kwargs['ordered'] = kwargs.get('ordered', self.ordered)
         ns = Namespace(*args, **kwargs)
         self.add_namespace(ns)
         return ns
@@ -469,9 +473,25 @@ class Api(object):
         :returns dict: the schema as a serializable dict
         '''
         if not self._schema:
-            self._schema = Swagger(self).as_dict()
+            try:
+                self._schema = Swagger(self).as_dict()
+            except Exception:
+                # Log the source exception for debugging purpose
+                # and return an error message
+                msg = 'Unable to render schema'
+                log.exception(msg)  # This will provide a full traceback
+                return {'error': msg}
         return self._schema
 
+    @property
+    def _own_and_child_error_handlers(self):
+        rv = {}
+        rv.update(self.error_handlers)
+        for ns in self.namespaces:
+            for exception, handler in six.iteritems(ns.error_handlers):
+                rv[exception] = handler
+        return rv
+
     def errorhandler(self, exception):
         '''A decorator to register an error handler for a given exception'''
         if inspect.isclass(exception) and issubclass(exception, Exception):
@@ -522,7 +542,7 @@ class Api(object):
             return self.owns_endpoint(rule.endpoint)
         except NotFound:
             return self.catch_all_404s
-        except:
+        except Exception:
             # Werkzeug throws other kinds of exceptions, such as Redirect
             pass
 
@@ -566,40 +586,51 @@ class Api(object):
         '''
         got_request_exception.send(current_app._get_current_object(), exception=e)
 
+        include_message_in_response = current_app.config.get("ERROR_INCLUDE_MESSAGE", True)
+        default_data = {}
+
         headers = Headers()
-        if e.__class__ in self.error_handlers:
-            handler = self.error_handlers[e.__class__]
-            result = handler(e)
-            default_data, code, headers = unpack(result, 500)
-        elif isinstance(e, HTTPException):
-            code = e.code
-            default_data = {
-                'message': getattr(e, 'description', HTTP_STATUS_CODES.get(code, ''))
-            }
-            headers = e.get_response().headers
-        elif self._default_error_handler:
-            result = self._default_error_handler(e)
-            default_data, code, headers = unpack(result, 500)
+
+        for typecheck, handler in six.iteritems(self._own_and_child_error_handlers):
+            if isinstance(e, typecheck):
+                result = handler(e)
+                default_data, code, headers = unpack(result, HTTPStatus.INTERNAL_SERVER_ERROR)
+                break
         else:
-            code = 500
-            default_data = {
-                'message': HTTP_STATUS_CODES.get(code, str(e)),
-            }
+            if isinstance(e, HTTPException):
+                code = HTTPStatus(e.code)
+                if include_message_in_response:
+                    default_data = {
+                        'message': getattr(e, 'description', code.phrase)
+                    }
+                headers = e.get_response().headers
+            elif self._default_error_handler:
+                result = self._default_error_handler(e)
+                default_data, code, headers = unpack(result, HTTPStatus.INTERNAL_SERVER_ERROR)
+            else:
+                code = HTTPStatus.INTERNAL_SERVER_ERROR
+                if include_message_in_response:
+                    default_data = {
+                        'message': code.phrase,
+                    }
+
+        if include_message_in_response:
+            default_data['message'] = default_data.get('message', str(e))
 
-        default_data['message'] = default_data.get('message', str(e))
         data = getattr(e, 'data', default_data)
         fallback_mediatype = None
 
-        if code >= 500:
+        if code >= HTTPStatus.INTERNAL_SERVER_ERROR:
             exc_info = sys.exc_info()
             if exc_info[1] is None:
                 exc_info = None
             current_app.log_exception(exc_info)
 
-        elif code == 404 and current_app.config.get("ERROR_404_HELP", True):
+        elif code == HTTPStatus.NOT_FOUND and current_app.config.get("ERROR_404_HELP", True) \
+                and include_message_in_response:
             data['message'] = self._help_on_404(data.get('message', None))
 
-        elif code == 406 and self.default_mediatype is None:
+        elif code == HTTPStatus.NOT_ACCEPTABLE and self.default_mediatype is None:
             # if we are handling NotAcceptable (406), make sure that
             # make_response uses a representation we support as the
             # default mediatype (so that make_response doesn't throw
@@ -613,7 +644,7 @@ class Api(object):
 
         resp = self.make_response(data, code, headers, fallback_mediatype=fallback_mediatype)
 
-        if code == 401:
+        if code == HTTPStatus.UNAUTHORIZED:
             resp = self.unauthorized(resp)
         return resp
 
@@ -768,7 +799,8 @@ class Api(object):
 class SwaggerView(Resource):
     '''Render the Swagger specifications as JSON'''
     def get(self):
-        return self.api.__schema__
+        schema = self.api.__schema__
+        return schema, HTTPStatus.INTERNAL_SERVER_ERROR if 'error' in schema else HTTPStatus.OK
 
     def mediatypes(self):
         return ['application/json']
@@ -776,9 +808,9 @@ class SwaggerView(Resource):
 
 def mask_parse_error_handler(error):
     '''When a mask can't be parsed'''
-    return {'message': 'Mask parse error: {0}'.format(error)}, 400
+    return {'message': 'Mask parse error: {0}'.format(error)}, HTTPStatus.BAD_REQUEST
 
 
 def mask_error_handler(error):
     '''When any error occurs on mask'''
-    return {'message': 'Mask error: {0}'.format(error)}, 400
+    return {'message': 'Mask error: {0}'.format(error)}, HTTPStatus.BAD_REQUEST
