@@ -1,17 +1,19 @@
 # -*- coding: utf-8 -*-
-
+from __future__ import unicode_literals
 
 import inspect
 import six
 import warnings
 
+from flask import request
 from flask.views import http_method_funcs
 
 from .errors import abort
 from .marshalling import marshal, marshal_with
-from .model import Model, SchemaModel
+from .model import Model, OrderedModel, SchemaModel
 from .reqparse import RequestParser
 from .utils import merge
+from ._http import HTTPStatus
 
 
 class Namespace(object):
@@ -25,9 +27,11 @@ class Namespace(object):
     :param str path: An optional prefix path. If not provided, prefix is ``/+name``
     :param list decorators: A list of decorators to apply to each resources
     :param bool validate: Whether or not to perform validation on this namespace
+    :param bool ordered: Whether or not to preserve order on models and marshalling
     :param Api api: an optional API to attache to the namespace
     '''
-    def __init__(self, name, description=None, path=None, decorators=None, validate=None, **kwargs):
+    def __init__(self, name, description=None, path=None, decorators=None, validate=None,
+            authorizations=None, ordered=False, **kwargs):
         self.name = name
         self.description = description
         self._path = path
@@ -40,10 +44,12 @@ class Namespace(object):
         self.resources = []
         self.error_handlers = {}
         self.default_error_handler = None
+        self.authorizations = authorizations
+        self.ordered = ordered
         self.apis = []
         if 'api' in kwargs:
             self.apis.append(kwargs['api'])
-            
+
     @property
     def path(self):
         return (self._path or ('/' + self.name)).rstrip('/')
@@ -138,7 +144,8 @@ class Namespace(object):
 
         .. seealso:: :class:`Model`
         '''
-        model = Model(name, model, mask=mask)
+        cls = OrderedModel if self.ordered else Model
+        model = cls(name, model, mask=mask)
         model.__apidoc__.update(kwargs)
         return self.add_model(name, model)
 
@@ -212,7 +219,7 @@ class Namespace(object):
         field.__apidoc__ = merge(getattr(field, '__apidoc__', {}), {'as_list': True})
         return field
 
-    def marshal_with(self, fields, as_list=False, code=200, description=None, **kwargs):
+    def marshal_with(self, fields, as_list=False, code=HTTPStatus.OK, description=None, **kwargs):
         '''
         A decorator specifying the fields to use for serialization.
 
@@ -228,7 +235,7 @@ class Namespace(object):
                 '__mask__': kwargs.get('mask', True),  # Mask values can't be determined outside app context
             }
             func.__apidoc__ = merge(getattr(func, '__apidoc__', {}), doc)
-            return marshal_with(fields, **kwargs)(func)
+            return marshal_with(fields, ordered=self.ordered, **kwargs)(func)
         return wrapper
 
     def marshal_list_with(self, fields, **kwargs):
@@ -274,7 +281,7 @@ class Namespace(object):
         :param ModelBase model: an optional response model
 
         '''
-        return self.doc(responses={code: (description, model) if model else description})
+        return self.doc(responses={code: (description, model, kwargs)})
 
     def header(self, name, description=None, **kwargs):
         '''
@@ -284,12 +291,36 @@ class Namespace(object):
         :param str description: a description about the header
 
         '''
-        return self.param(name, description=description, _in='header', **kwargs)
+        header = {'description': description}
+        header.update(kwargs)
+        return self.doc(headers={name: header})
+
+    def produces(self, mimetypes):
+        '''A decorator to specify the MIME types the API can produce'''
+        return self.doc(produces=mimetypes)
 
     def deprecated(self, func):
         '''A decorator to mark a resource or a method as deprecated'''
         return self.doc(deprecated=True)(func)
 
+    def vendor(self, *args, **kwargs):
+        '''
+        A decorator to expose vendor extensions.
+
+        Extensions can be submitted as dict or kwargs.
+        The ``x-`` prefix is optionnal and will be added if missing.
+
+        See: http://swagger.io/specification/#specification-extensions-128
+        '''
+        for arg in args:
+            kwargs.update(arg)
+        return self.doc(vendor=kwargs)
+
+    @property
+    def payload(self):
+        '''Store the input payload in the current request context'''
+        return request.get_json()
+
 
 def unshortcut_params_description(data):
     if 'params' in data:
