@@ -14,6 +14,7 @@ from werkzeug import exceptions
 from .errors import abort, SpecsError
 from .marshalling import marshal
 from .model import Model
+from ._http import HTTPStatus
 
 
 class ParseResult(dict):
@@ -29,6 +30,7 @@ class ParseResult(dict):
     def __setattr__(self, name, value):
         self[name] = value
 
+
 _friendly_location = {
     'json': 'the JSON body',
     'form': 'the post body',
@@ -49,7 +51,7 @@ LOCATIONS = {
     'files': 'formData',
 }
 
-#: Maps Pyton primitives types to Swagger ones
+#: Maps Python primitives types to Swagger ones
 PY_TYPES = {
     int: 'integer',
     str: 'string',
@@ -58,7 +60,9 @@ PY_TYPES = {
     None: 'void'
 }
 
-text_type = lambda x: six.text_type(x)
+SPLIT_CHAR = ','
+
+text_type = lambda x: six.text_type(x)  # noqa
 
 
 class Argument(object):
@@ -171,12 +175,12 @@ class Argument(object):
             bundled
         '''
         error_str = six.text_type(error)
-        error_msg = self.help.format(error_msg=error_str) if self.help else error_str
+        error_msg = ' '.join([six.text_type(self.help), error_str]) if self.help else error_str
         errors = {self.name: error_msg}
 
         if bundle_errors:
-            return error, errors
-        abort(400, 'Input payload validation failed', errors=errors)
+            return ValueError(error), errors
+        abort(HTTPStatus.BAD_REQUEST, 'Input payload validation failed', errors=errors)
 
     def parse(self, request, bundle_errors=False):
         '''
@@ -216,17 +220,18 @@ class Argument(object):
                             self.choices = [choice.lower() for choice in self.choices]
 
                     try:
-                        value = self.convert(value, operator)
+                        if self.action == 'split':
+                            value = [self.convert(v, operator) for v in value.split(SPLIT_CHAR)]
+                        else:
+                            value = self.convert(value, operator)
                     except Exception as error:
                         if self.ignore:
                             continue
                         return self.handle_validation_error(error, bundle_errors)
 
                     if self.choices and value not in self.choices:
-                        msg = '{0} is not a valid choice'.format(value)
-                        if bundle_errors:
-                            return self.handle_validation_error(msg, bundle_errors)
-                        self.handle_validation_error(msg, bundle_errors)
+                        msg = 'The value \'{0}\' is not a valid choice for \'{1}\'.'.format(value, name)
+                        return self.handle_validation_error(msg, bundle_errors)
 
                     if name in request.unparsed_arguments:
                         request.unparsed_arguments.pop(name)
@@ -239,9 +244,7 @@ class Argument(object):
                 locations = [_friendly_location.get(loc, loc) for loc in self.location]
                 location = ' or '.join(locations)
             error_msg = 'Missing required parameter in {0}'.format(location)
-            if bundle_errors:
-                return self.handle_validation_error(error_msg, bundle_errors)
-            self.handle_validation_error(error_msg, bundle_errors)
+            return self.handle_validation_error(error_msg, bundle_errors)
 
         if not results:
             if callable(self.default):
@@ -269,12 +272,16 @@ class Argument(object):
             param['required'] = True
         if self.help:
             param['description'] = self.help
-        if self.default:
+        if self.default is not None:
             param['default'] = self.default() if callable(self.default) else self.default
         if self.action == 'append':
             param['items'] = {'type': param['type']}
             param['type'] = 'array'
             param['collectionFormat'] = 'multi'
+        if self.action == 'split':
+            param['items'] = {'type': param['type']}
+            param['type'] = 'array'
+            param['collectionFormat'] = 'csv'
         if self.choices:
             param['enum'] = self.choices
             param['collectionFormat'] = 'multi'
@@ -354,7 +361,7 @@ class RequestParser(object):
             if found or arg.store_missing:
                 result[arg.dest or arg.name] = value
         if errors:
-            abort(400, message=errors)
+            abort(HTTPStatus.BAD_REQUEST, 'Input payload validation failed', errors=errors)
 
         if strict and req.unparsed_arguments:
             arguments = ', '.join(req.unparsed_arguments.keys())
