@@ -1,21 +1,21 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals, absolute_import
 
+import itertools
 import re
 
 from inspect import isclass, getdoc
-from collections import Hashable
+from collections import OrderedDict, Hashable
 from six import string_types, itervalues, iteritems, iterkeys
 
 from flask import current_app
 from werkzeug.routing import parse_rule
 
-from ._compat import OrderedDict
-
 from . import fields
 from .model import Model, ModelBase
 from .reqparse import RequestParser
 from .utils import merge, not_none, not_none_sorted
+from ._http import HTTPStatus
 
 
 #: Maps Flask/Werkzeug rooting types to Swagger ones
@@ -27,9 +27,10 @@ PATH_TYPES = {
 }
 
 
-#: Maps Pyton primitives types to Swagger ones
+#: Maps Python primitives types to Swagger ones
 PY_TYPES = {
     int: 'integer',
+    float: 'number',
     str: 'string',
     bool: 'boolean',
     None: 'void'
@@ -88,15 +89,23 @@ def extract_path_params(path):
 def _param_to_header(param):
     param.pop('in', None)
     param.pop('name', None)
+    return _clean_header(param)
 
-    typedef = param.get('type', 'string')
+
+def _clean_header(header):
+    if isinstance(header, string_types):
+        header = {'description': header}
+    typedef = header.get('type', 'string')
     if isinstance(typedef, Hashable) and typedef in PY_TYPES:
-        param['type'] = PY_TYPES[typedef]
+        header['type'] = PY_TYPES[typedef]
+    elif isinstance(typedef, (list, tuple)) and len(typedef) == 1 and typedef[0] in PY_TYPES:
+        header['type'] = 'array'
+        header['items'] = {'type': PY_TYPES[typedef[0]]}
     elif hasattr(typedef, '__schema__'):
-        param.update(typedef.__schema__)
+        header.update(typedef.__schema__)
     else:
-        param['type'] = typedef
-    return param
+        header['type'] = typedef
+    return not_none(header)
 
 
 def parse_docstring(obj):
@@ -167,6 +176,13 @@ class Swagger(object):
                 for url in self.api.ns_urls(ns, urls):
                     paths[extract_path(url)] = self.serialize_resource(ns, resource, url, kwargs)
 
+        # merge in the top-level authorizations
+        for ns in self.api.namespaces:
+            if ns.authorizations:
+                if self.api.authorizations is None:
+                    self.api.authorizations = {}
+                self.api.authorizations = merge(self.api.authorizations, ns.authorizations)
+
         specs = {
             'swagger': '2.0',
             'basePath': basepath,
@@ -233,7 +249,7 @@ class Swagger(object):
                 method_doc['docstring'] = parse_docstring(method_impl)
                 method_params = self.expected_params(method_doc)
                 method_params = merge(method_params, method_doc.get('params', {}))
-                inherited_params = dict((k, v) for k, v in iteritems(params) if k in method_params)
+                inherited_params = OrderedDict((k, v) for k, v in iteritems(params) if k in method_params)
                 method_doc['params'] = merge(inherited_params, method_params)
             doc[method] = method_doc
         return doc
@@ -245,7 +261,7 @@ class Swagger(object):
 
         for expect in doc.get('expect', []):
             if isinstance(expect, RequestParser):
-                parser_params = dict((p['name'], p) for p in expect.__schema__)
+                parser_params = OrderedDict((p['name'], p) for p in expect.__schema__)
                 params.update(parser_params)
             elif isinstance(expect, ModelBase):
                 params['payload'] = not_none({
@@ -276,17 +292,13 @@ class Swagger(object):
 
     def register_errors(self):
         responses = {}
-        for exception, handler in self.api.error_handlers.items():
+        for exception, handler in iteritems(self.api.error_handlers):
             doc = parse_docstring(handler)
             response = {
                 'description': doc['summary']
             }
             apidoc = getattr(handler, '__apidoc__', {})
-            if 'params' in apidoc:
-                response['headers'] = dict(
-                    (n, _param_to_header(o))
-                    for n, o in apidoc['params'].items() if o.get('in') == 'header'
-                )
+            self.process_headers(response, apidoc)
             if 'responses' in apidoc:
                 _, model = list(apidoc['responses'].values())[0]
                 response['schema'] = self.serialize_schema(model)
@@ -317,6 +329,9 @@ class Swagger(object):
             'parameters': self.parameters_for(doc[method]) or None,
             'security': self.security_for(doc, method),
         }
+        # Handle 'produces' mimetypes documentation
+        if 'produces' in doc[method]:
+            operation['produces'] = doc[method]['produces']
         # Handle deprecated annotation
         if doc.get('deprecated') or doc[method].get('deprecated'):
             operation['deprecated'] = True
@@ -326,8 +341,20 @@ class Swagger(object):
                 operation['consumes'] = ['multipart/form-data']
             else:
                 operation['consumes'] = ['application/x-www-form-urlencoded', 'multipart/form-data']
+        operation.update(self.vendor_fields(doc, method))
         return not_none(operation)
 
+    def vendor_fields(self, doc, method):
+        '''
+        Extract custom 3rd party Vendor fields prefixed with ``x-``
+
+        See: http://swagger.io/specification/#specification-extensions-128
+        '''
+        return dict(
+            (k if k.startswith('x-') else 'x-{0}'.format(k), v)
+            for k, v in iteritems(doc[method].get('vendor', {}))
+        )
+
     def description_for(self, doc, method):
         '''Extract the description metadata and fallback on the whole docstring'''
         parts = []
@@ -388,7 +415,17 @@ class Swagger(object):
         for d in doc, doc[method]:
             if 'responses' in d:
                 for code, response in iteritems(d['responses']):
-                    description, model = (response, None) if isinstance(response, string_types) else response
+                    if isinstance(response, string_types):
+                        description = response
+                        model = None
+                        kwargs = {}
+                    elif len(response) == 3:
+                        description, model, kwargs = response
+                    elif len(response) == 2:
+                        description, model = response
+                        kwargs = {}
+                    else:
+                        raise ValueError('Unsupported response specification')
                     description = description or DEFAULT_RESPONSE_DESCRIPTION
                     if code in responses:
                         responses[code].update(description=description)
@@ -396,15 +433,16 @@ class Swagger(object):
                         responses[code] = {'description': description}
                     if model:
                         responses[code]['schema'] = self.serialize_schema(model)
+                    self.process_headers(responses[code], doc, method, kwargs.get('headers'))
             if 'model' in d:
-                code = str(d.get('default_code', 200))
+                code = str(d.get('default_code', HTTPStatus.OK))
                 if code not in responses:
-                    responses[code] = DEFAULT_RESPONSE.copy()
+                    responses[code] = self.process_headers(DEFAULT_RESPONSE.copy(), doc, method)
                 responses[code]['schema'] = self.serialize_schema(d['model'])
 
             if 'docstring' in d:
-                for name, description in d['docstring']['raises'].items():
-                    for exception, handler in self.api.error_handlers.items():
+                for name, description in iteritems(d['docstring']['raises']):
+                    for exception, handler in iteritems(self.api.error_handlers):
                         error_responses = getattr(handler, '__apidoc__', {}).get('responses', {})
                         code = list(error_responses.keys())[0] if error_responses else None
                         if code and exception.__name__ == name:
@@ -412,9 +450,22 @@ class Swagger(object):
                             break
 
         if not responses:
-            responses['200'] = DEFAULT_RESPONSE.copy()
+            responses[str(HTTPStatus.OK.value)] = self.process_headers(DEFAULT_RESPONSE.copy(), doc, method)
         return responses
 
+    def process_headers(self, response, doc, method=None, headers=None):
+        method_doc = doc.get(method, {})
+        if 'headers' in doc or 'headers' in method_doc or headers:
+            response['headers'] = dict(
+                (k, _clean_header(v)) for k, v
+                in itertools.chain(
+                    iteritems(doc.get('headers', {})),
+                    iteritems(method_doc.get('headers', {})),
+                    iteritems(headers or {})
+                )
+            )
+        return response
+
     def serialize_definitions(self):
         return dict(
             (name, model.__schema__)
